'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.Field = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _provider = require('./provider');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var INTEGER_REGEX = /^-?\d+?$/;
var FLOAT_REGEX = /^-?\d+(\.\d+)?$/;
var EMAIL_REGEX = /\S+@\S+\.\S+/;

var Field = exports.Field = function (_React$Component) {
    _inherits(Field, _React$Component);

    function Field(props) {
        _classCallCheck(this, Field);

        var _this = _possibleConstructorReturn(this, (Field.__proto__ || Object.getPrototypeOf(Field)).call(this, props));

        _this.state = {
            valid: true,
            invalid: false,
            untouched: true,
            touched: false,
            pristine: true,
            dirty: false,
            unchanged: true,
            changed: false,
            errors: [],
            value: '',
            initialValue: props.initialValue
        };

        _this.handleFieldChangeForward = _this.handleFieldChangeForward.bind(_this);
        _this.handleFormResetForward = _this.handleFormResetForward.bind(_this);
        _this.handleChange = _this.handleChange.bind(_this);
        _this.handleBlur = _this.handleBlur.bind(_this);
        _this.updateFieldCtrl = _this.updateFieldCtrl.bind(_this);
        _this.getChildProps = _this.getChildProps.bind(_this);
        _this.transformProps = _this.transformProps.bind(_this);
        _this.sync = _this.sync.bind(_this);
        return _this;
    }

    _createClass(Field, [{
        key: 'componentWillMount',
        value: function componentWillMount() {
            var _props = this.props,
                form = _props.form,
                name = _props.name,
                children = _props.children,
                _props$initialValue = _props.initialValue,
                initialValue = _props$initialValue === undefined ? '' : _props$initialValue;

            if (Array.isArray(children) && children.length > 1) {
                throw 'The Field component for "' + form + '#' + name + '" should have only one child, but has ' + children.length + '.';
            }
            document.addEventListener(_provider.REACT_FORMCTRL.EVENTS.FIELD_CHANGED + '#' + form + '#' + name, this.handleFieldChangeForward);
            document.addEventListener(_provider.REACT_FORMCTRL.EVENTS.FORM_RESETED + '#' + form, this.handleFormResetForward);

            var fieldCtrl = this.updateFieldCtrl(initialValue);
            fieldCtrl.value = initialValue;
            _provider.FormEventDispatcher.dispatchRegisterField(form, name, fieldCtrl);
        }
    }, {
        key: 'componentWillUnmount',
        value: function componentWillUnmount() {
            var _props2 = this.props,
                form = _props2.form,
                name = _props2.name;

            document.removeEventListener(_provider.REACT_FORMCTRL.EVENTS.FIELD_CHANGED + '#' + form + '#' + name, this.handleFieldChangeForward);
            document.removeEventListener(_provider.REACT_FORMCTRL.EVENTS.FORM_RESETED + '#' + form, this.handleFormResetForward);
            _provider.FormEventDispatcher.dispatchUnregisterField(form, name);
        }
    }, {
        key: 'updateFieldCtrl',
        value: function updateFieldCtrl(value) {
            var errors = [];
            var _props3 = this.props,
                form = _props3.form,
                name = _props3.name,
                type = _props3.type,
                required = _props3.required,
                pattern = _props3.pattern,
                match = _props3.match,
                integer = _props3.integer,
                min = _props3.min,
                max = _props3.max,
                minLength = _props3.minLength,
                maxLength = _props3.maxLength,
                _props3$initialValue = _props3.initialValue,
                initialValue = _props3$initialValue === undefined ? '' : _props3$initialValue;

            if (required && !value) errors.push('required');else if (pattern && !new RegExp(pattern).test(value)) errors.push('pattern');else {
                if (type === 'email' && !EMAIL_REGEX.test(value)) errors.push('email');
                if (type === 'number') {
                    if (integer && !INTEGER_REGEX.test(value)) errors.push('integer');
                    if (!integer && !FLOAT_REGEX.test(value)) errors.push('float');
                    if (min && +value < min) errors.push('min');
                    if (max && +value > max) errors.push('max');
                } else {
                    if (minLength && value && value.length < minLength) errors.push('minLength');
                    if (maxLength && value && value.length > maxLength) errors.push('maxLength');
                }
            }
            return _extends({}, this.state, {
                errors: errors,
                unchanged: value === initialValue,
                changed: value !== initialValue,
                valid: errors.length === 0,
                invalid: errors.length > 0
            });
        }
    }, {
        key: 'handleFieldChangeForward',
        value: function handleFieldChangeForward(event) {
            var payload = event.detail;
            var fieldCtrl = payload.fieldCtrl;
            var syncState = this.sync(fieldCtrl);
            if (Object.keys(syncState).length) {
                this.setState(syncState);
            }
        }
    }, {
        key: 'handleFormResetForward',
        value: function handleFormResetForward(event) {
            var _props4 = this.props,
                form = _props4.form,
                name = _props4.name,
                children = _props4.children,
                _props4$initialValue = _props4.initialValue,
                initialValue = _props4$initialValue === undefined ? '' : _props4$initialValue;

            var fieldCtrl = this.updateFieldCtrl(initialValue);
            fieldCtrl.value = initialValue;
            fieldCtrl.pristine = true;
            fieldCtrl.dirty = false;
            fieldCtrl.untouched = true;
            fieldCtrl.touched = false;
            _provider.FormEventDispatcher.dispatchFieldChanged(form, name, fieldCtrl);
        }
    }, {
        key: 'handleChange',
        value: function handleChange(event) {
            var updateFieldCtrl = this.updateFieldCtrl,
                sync = this.sync;
            var _props5 = this.props,
                form = _props5.form,
                name = _props5.name;

            var value = event.target.value;
            var fieldCtrl = updateFieldCtrl(value);
            fieldCtrl.value = value;
            fieldCtrl.pristine = false;
            fieldCtrl.dirty = true;
            var syncState = sync(fieldCtrl);
            if (Object.keys(syncState).length) {
                _provider.FormEventDispatcher.dispatchFieldChanged(form, name, fieldCtrl);
            }
        }
    }, {
        key: 'handleBlur',
        value: function handleBlur(event) {
            if (this.state.untouched) {
                var _props6 = this.props,
                    form = _props6.form,
                    name = _props6.name;

                _provider.FormEventDispatcher.dispatchFieldChanged(form, name, _extends({}, this.state, { touched: true, untouched: false }));
            }
        }
    }, {
        key: 'sync',
        value: function sync(fieldCtrl) {
            var syncState = {};
            if (JSON.stringify(this.state.errors) !== JSON.stringify(fieldCtrl.errors)) syncState.errors = fieldCtrl.errors;
            if (this.state.valid !== fieldCtrl.valid) syncState.valid = fieldCtrl.valid;
            if (this.state.invalid !== fieldCtrl.invalid) syncState.invalid = fieldCtrl.invalid;
            if (this.state.untouched !== fieldCtrl.untouched) syncState.untouched = fieldCtrl.untouched;
            if (this.state.touched !== fieldCtrl.touched) syncState.touched = fieldCtrl.touched;
            if (this.state.pristine !== fieldCtrl.pristine) syncState.pristine = fieldCtrl.pristine;
            if (this.state.dirty !== fieldCtrl.dirty) syncState.dirty = fieldCtrl.dirty;
            if (this.state.unchanged !== fieldCtrl.unchanged) syncState.unchanged = fieldCtrl.unchanged;
            if (this.state.changed !== fieldCtrl.changed) syncState.changed = fieldCtrl.changed;
            if (this.state.value !== fieldCtrl.value) syncState.value = fieldCtrl.value;
            if (this.state.__instances != fieldCtrl.__instances) syncState.__instances = fieldCtrl.__instances;
            return syncState;
        }
    }, {
        key: 'getChildProps',
        value: function getChildProps() {
            var props = {};
            props.name = this.props.name;
            props.ctrl = {
                valid: this.state.valid,
                invalid: this.state.invalid,
                untouched: this.state.untouched,
                touched: this.state.touched,
                pristine: this.state.pristine,
                dirty: this.state.dirty,
                unchanged: this.state.unchanged,
                changed: this.state.changed,
                errors: this.state.errors
            };
            props.className = this.props.className;
            props.value = this.state.value;
            props.required = this.props.required;
            props.pattern = this.props.pattern;
            props.type = this.props.type || 'text';
            props.onChange = this.handleChange;
            props.onBlur = this.handleBlur;
            return props;
        }
    }, {
        key: 'transformProps',
        value: function transformProps(childProps) {
            var transformProps = this.props.transformProps;

            var props = this.getChildProps();
            if (typeof transformProps === 'function') {
                return transformProps(props);
            }
            return props;
        }
    }, {
        key: 'render',
        value: function render() {
            var getChildProps = this.getChildProps,
                transformProps = this.transformProps;
            var children = this.props.children;

            var child = children;
            if (Array.isArray(children)) {
                child = children[0];
            }
            return _react2.default.cloneElement(child, _extends({}, child.props, transformProps()));
        }
    }]);

    return Field;
}(_react2.default.Component);